///////////////////////////////////////////
// Generic functions (normally in a LIB) //
///////////////////////////////////////////

FUNCTION DrawBox( nTopRow, nLeftCol, nBotRow, nRightCol, cBorder )
* Save screen and draw box in one step.
LOCAL cSaveScrn := Savebox( nTopRow, nLeftCol, nBotRow, nRightCol )
cBorder := IF( cBorder == NIL, "Ŀ ", cBorder )
DISPBOX( nTopRow, nLeftCol, nBotRow, nRightCol, cBorder )
RETURN cSaveScrn


FUNCTION SaveBox( nTopRow, nLeftCol, nBotRow, nRightCol )
* Save all pertinant information into one character string!
RETURN CHR(nTopRow) + CHR(nLeftCol) + CHR(nBotRow) + CHR(nRightCol)+;
       CHR(ROW()) + CHR(COL()) + CHR(SETCURSOR())+;
       SAVESCREEN(nTopRow, nLeftCol, nBotRow, nRightCol)


FUNCTION RestBox( cPopSaved )
LOCAL nTopRow, nLeftCol, nBotRow, nRightCol, nCursRow, nCursCol, nCursType
* Screen positions are stored as ASCII characters at beginning of string.
nTopRow   := ASC( SUBSTR( cPopSaved, 1, 1) )
nLeftCol  := ASC( SUBSTR( cPopSaved, 2, 1) )
nBotRow   := ASC( SUBSTR( cPopSaved, 3, 1) )
nRightCol := ASC( SUBSTR( cPopSaved, 4, 1) )
nCursRow  := ASC( SUBSTR( cPopSaved, 5, 1) )
nCursCol  := ASC( SUBSTR( cPopSaved, 6, 1) )
nCursType := ASC( SUBSTR( cPopSaved, 7, 1) )
cPopSaved := SUBSTR( cPopSaved, 8, LEN(cPopSaved)-7 )
Restscreen( nTopRow, nLeftCol, nBotRow, nRightCol, cPopSaved )
Setpos( nCursRow, nCursCol )   // put the cursor back where it was...
Setcursor( nCursType )         // reset the cursor to previous state...
RETURN nil


FUNCTION Between( TheValue, TheMin, TheMax )
RETURN TheValue >= TheMin .AND. TheValue <= TheMax


/*
Program:  STATUSBA.PRG
 Author:  Stephen L. Woolstenhulme
Purpose:  Display a 'status of completion' box and bar.

 Syntax:  StatusBar( < nBytes >, < nSize >, [< nDispRow >],
          [< cTitle >], [< cColor >], [< lTwirl >]  )--> .t.

 Params:  nBytes   - Current recno() or bytes completed.
                     0 to initialize.
          nSize    - The total number of bytes/records.
          nDispRow - Top row to display status box.
                     Defaults to maxrow() - 4.
          cTitle   - The title for this status bar.
                     Defaults to 'Percentage Complete'.
          cColor   - The color for the status bar.
          lTwirl   - Show the twirling baton?

Returns: .t.

Example:  Note -- Call StatusBar() once to set up, again to do the work,
                  and a third time to clean things up.

// Initialize everything.
StatusBar( 0, reccount(), 18, 'Indexing by Name', 'W+/R' )

// Build the new index, calling StatusBar in the code block.
dbcreateindex( 'BY_NAME.NTX', 'upper( LAST + FIRST )', ;
      { || StatusBar( recno() ), upper( LAST + FIRST ) } )

// Clear everything off.
StatusBar(0)

10/07/92 - Modified the display of percentages and the bar used. (JDW)
           Now the system shows half-bar progress too...
10/09/92 - Added twirling baton to show additionl progress.      (JDW)
11/06/92 - Make sure baton is not show at 100%                   (JDW)
07/23/93 - Make the baton an optional parameter.                 (JDW)

*/

#include 'setcurs.ch'

function StatusBar( nBytes, nSize, nDispRow, cTitle, cColor, lTwirl )
  STATIC lBox, cScrn, cActiveColor, nMaxRow, nTotalSize, nKeyState
  STATIC nSpin := 1, lBaton := .T.
  LOCAL nReplicate, cSpin

  if lBox == NIL .or. nBytes == 0

    IF VALTYPE( lTwirl ) == "L"
       lBaton := lTwirl
    ENDIF

    if lBox != NIL .and. lBox
      // Restore screen, reset state variables.
      RestScreen( nMaxRow -4, 13 , nMaxRow - 1, 64, cScrn )
      lBox         := NIL
      cActiveColor := NIL
      nMaxRow      := NIL
      nTotalSize   := NIL
      SetCursor( nKeyState )
    else            

      if cActiveColor == NIL .or. cColor != NIL
        cActiveColor := if( cColor != NIL, cColor, setcolor( ) )
      endif                            

      if nMaxRow == NIL .or. nDispRow != NIL
        nMaxRow := if( nDispRow == NIL , maxRow() , nDispRow + 4 )
      endif

      if nTotalSize == NIL .or. nSize != NIL
         nTotalSize := if( nSize == NIL, 0, nSize )
      endif              

      nKeyState := SetCursor( SC_NONE )

      // Save and draw our box.
      cScrn := SaveScreen( nMaxRow -4, 13 , nMaxRow - 1, 64 )
      DispBox( nMaxRow -4, 13 , nMaxRow - 2, ;
               64, "͸Գ ", cActiveColor )

      @ nMaxRow-3,14 SAY "   10   20   30   40   50   60   70   80   90  100" ;
         COLOR cActiveColor
      
      // Write our title.
      if( cTitle == NIL, cTitle := 'Percentage Complete', )
      cTitle := '[ ' + cTitle + ' ]'
      @ nMaxRow-4, maxcol() / 2 - ( len( cTitle ) / 2 ) SAY cTitle COLOR cColor
      lBox := .t. 
    endif

  else  

    if nBytes <= nTotalSize
      nReplicate := INT( ( ( nBytes / nTotalSize ) * 100 ) / 2 )
      if lBaton .AND. nBytes <> nTotalSize
         cSpin := SUBSTR( "/\|" , nSpin, 1 )
      else
         cSpin := ""
      endif
      @ nMaxRow-3,14 SAY REPLICATE(CHR(219),nReplicate)+cSpin COLOR cActiveColor
      nSpin := IF( nSpin = 4, 1, nSpin+1 )
    endif

  endif

return .t.


/***
*  FWriteLn( <nHandle>, <cString>, [<nLength>], [<cDelim>] ) --> nBytes
*  Write a line to a text file at the current file pointer position. 
*  
*  NOTE: Check FERROR() for the error
*
*  NOTE: nLength defaults to length of entire string + delim, cDelim
*        defaults to CHR(13) + CHR(10)
*
*  NOTE: Return value includes length of delimiter
*
*/
FUNCTION FWriteLn( nHandle, cString, nLength, cDelim )

   IF cDelim == NIL
      cString += CHR(13) + CHR(10)
   ELSE
      cString += cDelim
   ENDIF

   RETURN FWRITE( nHandle, cString, nLength )

